﻿
using ProjectAirFighter.Exceptions;

namespace ProjectAirFighter.CollectionGenericObjects;

/// <summary>
/// Параметризованный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Массив объектов, которые храним
    /// </summary>
    private T?[] _collection;

    public int Count => _collection.Length;

    public int MaxCount
    {
        get
        {
            return _collection.Length;
        }
        set
        {
            if (value > 0)
            {
                if (_collection.Length > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.Massive;

    /// <summary>
    /// Конструктор
    /// </summary>

    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }

    public T? Get(int position)
    {
        if (position < 0 || position >= Count) throw new PositionOutOfCollectionException(position);
        if (_collection[position] == null) throw new ObjectNotFoundException(position);
        return _collection[position];
    }

    public int Insert(T obj, IEqualityComparer<T?>? comparer = null)
    {
        if (comparer != null)
        {
            foreach (T? i in _collection)
            {
                if (comparer.Equals(i, obj))
                {
                    throw new ObjectAlreadyExistsException(i);
                }
            }
        }
        return Insert(obj, 0);
    }

    public int Insert(T obj, int position, IEqualityComparer<T?>? comparer = null)
    {

        if (position < 0 || position >= Count)
        {
            throw new PositionOutOfCollectionException();
        }

        if (comparer != null)
        {
            foreach (T? i in _collection)
            {
                if (comparer.Equals(i, obj))
                {
                    throw new ObjectAlreadyExistsException(i);
                }
            }
        }

        if (_collection[position] != null)
        {
            bool pushed = false;
            for (int index = position + 1; index < Count; index++)
            {
                if (_collection[index] == null)
                {
                    position = index;
                    pushed = true;
                    break;
                }
            }

            if (!pushed)
            {
                for (int index = position - 1; index >= 0; index--)
                {
                    if (_collection[index] == null)
                    {
                        position = index;
                        pushed = true;
                        break;
                    }
                }
            }

            if (!pushed)
            {
                throw new CollectionOverflowException(Count);
            }
        }

        _collection[position] = obj;
        return position;
    }

    public T? Remove(int position)
    {
        if (position < 0 || position >= Count) throw new PositionOutOfCollectionException(position);
        if (_collection[position] == null) throw new ObjectNotFoundException(position);
        T? temp = _collection[position];
        _collection[position] = null;
        return temp;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Length; ++i)
        {
            yield return _collection[i];
        }
    }
    public void CollectionSort(IComparer<T?> comparer)
    {
        List<T?> lst = [.. _collection];
        lst.Sort(comparer.Compare);
        for (int i = 0; i < _collection.Length; ++i)
        {
            _collection[i] = lst[i];
        }
    }
}