﻿using ProjectAirFighter.Entities;

namespace ProjectAirFighter.Drawnings;
/// <summary>
/// Класс, отвечающий за прорисовку и перемещение базового объекта-сущности
/// </summary>
public class DrawningAirCraft
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityAirCraft? EntityAirCraft { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки самолёта
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки самолёта
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки самолёта
    /// </summary>
    private readonly int _drawningAirFighterWidth = 66;

    /// <summary>
    /// Высота прорисовки самолёта
    /// </summary>
    private readonly int _drawningAirFighterHeight = 74;

    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningAirFighterWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningAirFighterHeight;
    /// <summary>
    /// Пустой конструктор
    /// </summary>
    private DrawningAirCraft()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    public DrawningAirCraft(int speed, double weight, Color bodyColor) : this()
    {
        EntityAirCraft = new EntityAirCraft(speed, weight, bodyColor);

    }

    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawningAirFighterWidth">Ширина прорисовки самолёта</param>
    /// <param name="drawningAirFighterHeight">Высота прорисовки самолёта</param>
    protected DrawningAirCraft(int drawningAirFighterWidth, int drawningAirFighterHeight) : this()
    {
        _drawningAirFighterWidth = drawningAirFighterWidth;
        _drawningAirFighterHeight = drawningAirFighterHeight;
    }

    /// <summary>
    /// Конструктор для Extention
    /// </summary>
    /// <param name="airplan"></param>
    public DrawningAirCraft(EntityAirCraft aircraft) : this()
    {
        EntityAirCraft = new EntityAirCraft(aircraft.Speed, aircraft.Weight, aircraft.BodyColor);
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {

        if (_drawningAirFighterWidth > width || _drawningAirFighterHeight > height)
        {
            return false;
        }

        _pictureWidth = width;
        _pictureHeight = height;

        if (_startPosX.HasValue && _startPosY.HasValue)
        {
            SetPosition(_startPosX.Value, _startPosY.Value);
        }

        return true;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    /// 
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {

            return;
        }

        if (x < 0)
        {
            x = 0;
        }
        else if (x > _pictureWidth - _drawningAirFighterWidth)
        {
            x = _pictureWidth.Value - _drawningAirFighterWidth;
        }

        if (y < 0)
        {
            y = 0;
        }
        else if (y > _pictureHeight - _drawningAirFighterHeight)
        {
            y = _pictureHeight.Value - _drawningAirFighterHeight;
        }

        _startPosX = x;
        _startPosY = y;
    }

    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityAirCraft == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityAirCraft.Step > 0)
                {
                    _startPosX -= (int)EntityAirCraft.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityAirCraft.Step > 0)
                {
                    _startPosY -= (int)EntityAirCraft.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityAirCraft.Step < _pictureWidth - _drawningAirFighterWidth)
                {
                    _startPosX += (int)EntityAirCraft.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityAirCraft.Step < _pictureHeight - _drawningAirFighterHeight)
                {
                    _startPosY += (int)EntityAirCraft.Step;
                }
                return true;
            default:
                return false;
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityAirCraft == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);

        //фюзеляж
        Brush br = new SolidBrush(EntityAirCraft.BodyColor);

        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 25, 50, 24);
        g.FillRectangle(br, _startPosX.Value, _startPosY.Value + 25, 50, 24);

        Point[] nose = new Point[] {
            new Point { X = _startPosX.Value + 50, Y = _startPosY.Value + 25},
            new Point { X = _startPosX.Value + 66, Y = _startPosY.Value + 37},
            new Point { X = _startPosX.Value + 50, Y = _startPosY.Value + 49}
        };

        g.DrawPolygon(pen, nose);
        Brush brgray = new SolidBrush(Color.Gray);
        g.FillPolygon(brgray, nose);

        //левое крыло
        Point[] wingl = new Point[] {
            new Point { X = _startPosX.Value + 35, Y = _startPosY.Value + 25},
            new Point { X = _startPosX.Value + 35, Y = _startPosY.Value},
            new Point { X = _startPosX.Value + 27, Y = _startPosY.Value},
            new Point { X = _startPosX.Value + 20, Y = _startPosY.Value + 25}
        };

        g.DrawPolygon(pen, wingl);
        g.FillPolygon(br, wingl);

        //правое крыло
        Point[] wingr = new Point[] {
            new Point { X = _startPosX.Value + 35, Y = _startPosY.Value + 49},
            new Point { X = _startPosX.Value + 35, Y = _startPosY.Value + 74},
            new Point { X = _startPosX.Value + 27, Y = _startPosY.Value + 74},
            new Point { X = _startPosX.Value + 20, Y = _startPosY.Value + 49}
        };

        g.DrawPolygon(pen, wingr);
        g.FillPolygon(br, wingr);

        //левый хвост
        Point[] ltail = new Point[] {
            new Point { X = _startPosX.Value, Y = _startPosY.Value + 49},
            new Point { X = _startPosX.Value, Y = _startPosY.Value + 66},
            new Point { X = _startPosX.Value + 10, Y = _startPosY.Value + 57},
            new Point { X = _startPosX.Value + 10, Y = _startPosY.Value + 49}
        };

        g.DrawPolygon(pen, ltail);
        g.FillPolygon(br, ltail);

        //правый хвост
        Point[] rtail = new Point[] {
            new Point { X = _startPosX.Value, Y = _startPosY.Value + 25},
            new Point { X = _startPosX.Value, Y = _startPosY.Value + 8},
            new Point { X = _startPosX.Value + 10, Y = _startPosY.Value + 17},
            new Point { X = _startPosX.Value + 10, Y = _startPosY.Value + 25}
        };

        g.DrawPolygon(pen, rtail);
        g.FillPolygon(br, rtail);

        //обводка
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 25, 50, 24);
        g.DrawLine(pen, _startPosX.Value + 50, _startPosY.Value + 25, _startPosX.Value + 66, _startPosY.Value + 37);
        g.DrawLine(pen, _startPosX.Value + 66, _startPosY.Value + 37, _startPosX.Value + 50, _startPosY.Value + 49);

        g.DrawLine(pen, _startPosX.Value + 35, _startPosY.Value + 25, _startPosX.Value + 35, _startPosY.Value);
        g.DrawLine(pen, _startPosX.Value + 35, _startPosY.Value, _startPosX.Value + 27, _startPosY.Value);
        g.DrawLine(pen, _startPosX.Value + 27, _startPosY.Value, _startPosX.Value + 20, _startPosY.Value + 25);

        g.DrawLine(pen, _startPosX.Value + 35, _startPosY.Value + 49, _startPosX.Value + 35, _startPosY.Value + 74);
        g.DrawLine(pen, _startPosX.Value + 35, _startPosY.Value + 74, _startPosX.Value + 27, _startPosY.Value + 74);
        g.DrawLine(pen, _startPosX.Value + 27, _startPosY.Value + 74, _startPosX.Value + 20, _startPosY.Value + 49);

        g.DrawLine(pen, _startPosX.Value, _startPosY.Value + 49, _startPosX.Value, _startPosY.Value + 66);
        g.DrawLine(pen, _startPosX.Value, _startPosY.Value + 66, _startPosX.Value + 10, _startPosY.Value + 57);
        g.DrawLine(pen, _startPosX.Value + 10, _startPosY.Value + 57, _startPosX.Value + 10, _startPosY.Value + 49);

        g.DrawLine(pen, _startPosX.Value, _startPosY.Value + 25, _startPosX.Value, _startPosY.Value + 8);
        g.DrawLine(pen, _startPosX.Value, _startPosY.Value + 8, _startPosX.Value + 10, _startPosY.Value + 17);
        g.DrawLine(pen, _startPosX.Value + 10, _startPosY.Value + 17, _startPosX.Value + 10, _startPosY.Value + 25);
    }
}
