﻿using Microsoft.Extensions.Logging;
using ProjectAirFighter.CollectionGenericObjects;
using ProjectAirFighter.Drawnings;
using ProjectAirFighter.Exceptions;

namespace ProjectAirFighter;

/// <summary>
/// Форма работы с компанией и ее коллекцией
/// </summary>
public partial class FormAirCraftCollection : Form
{
    /// <summary>
    /// Хранилише коллекций
    /// </summary>
    private readonly StorageCollection<DrawningAirCraft> _storageCollection;

    /// <summary>
    /// Компания
    /// </summary>
    private AbstractCompany? _company = null;

    /// <summary>
    /// Логер
    /// </summary>
    private readonly ILogger _logger;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormAirCraftCollection(ILogger<FormAirCraftCollection> logger)
    {
        InitializeComponent();
        _storageCollection = new();
        _logger = logger;
    }

    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }

    /// <summary>
    /// Добавление самолёта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddAirCraft_Click(object sender, EventArgs e)
    {
        FormAirCraftConfig form = new FormAirCraftConfig();
        form.Show();
        form.AddEvent(SetAirCraft);
    }

    /// <summary>
	/// Добавление самолёта в коллекцию
	/// </summary>
	/// <param name="aircraft"></param>
    private void SetAirCraft(DrawningAirCraft? aircraft)
    {
        if (_company == null || aircraft == null)
        {
            return;
        }
        try
        {
            int addingObject = (_company + aircraft);
            MessageBox.Show("Объект добавлен");
            _logger.LogInformation($"Добавлен объект {aircraft.GetDataForSave()}");
            pictureBox.Image = _company.Show();
        }
        catch (CollectionOverflowException ex)
        {
            MessageBox.Show("Не удалось добавить объект");
            _logger.LogError($"Не удалось добавить объект: {ex.Message}");
        }
        catch (ObjectAlreadyExistsException)
        {
            MessageBox.Show("Такой объект уже существует");
            _logger.LogError("Ошибка: такой объект уже существует {0}", aircraft);
        }
    }

    /// <summary>
    /// Удаление объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRemoveAirCraft_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
        {
            _logger.LogError("Удаление объекта из несуществующей коллекции");
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
        try
        {
            object decrementObject = _company - pos;
            MessageBox.Show("Объект удален");
            _logger.LogInformation($"Удален объект по позиции {pos}");
            pictureBox.Image = _company.Show();
        }
        catch (ObjectNotFoundException)
        {
            MessageBox.Show("Объект не найден");
            _logger.LogError($"Удаление не найденного объекта в позиции {pos} ");
        }
        catch (PositionOutOfCollectionException)
        {
            MessageBox.Show("Удаление вне рамках коллекции");
            _logger.LogError($"Удаление объекта за пределами коллекции {pos} ");
        }
    }

    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        try
        {
            DrawningAirCraft? aircraft = null;
            int counter = 100;
            while (aircraft == null)
            {
                aircraft = _company.GetRandomObject();
                counter--;
                if (counter <= 0)
                {
                    break;
                }
            }
            if (aircraft == null)
            {
                return;
            }
            FormAirFighter form = new()
            {
                SetAirCraft = aircraft
            };
            form.ShowDialog();
        }
        catch (ObjectNotFoundException)
        {
            _logger.LogError("Ошибка при передаче объекта на FormAirFighter");
        }
    }

    /// <summary>
    /// Перерисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }

    /// <summary>
    /// Добавление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            _logger.LogError("Не заполненная коллекция");
            return;
        }

        CollectionType collectionType = CollectionType.None;
        if (radioButtonMassive.Checked)
        {
            collectionType = CollectionType.Massive;
        }
        else if (radioButtonList.Checked)
        {
            collectionType = CollectionType.List;
        }

        _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
        _logger.LogInformation($"Добавлена коллекция: {textBoxCollectionName.Text}");
        RerfreshListBoxItems();
    }

    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionDel_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            _logger.LogError("Удаление невыбранной коллекции");
            return;
        }
        string name = listBoxCollection.SelectedItem.ToString() ?? string.Empty;
        if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }
        _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
        _logger.LogInformation($"Удалена коллекция: {name}");
        RerfreshListBoxItems();
    }

    /// <summary>
    /// Обновление списка в listBoxCollection
    /// </summary>
    private void RerfreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i].Name;
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }
    /// <summary>
    /// Создание компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            _logger.LogError("Создание компании невыбранной коллекции");
            return;
        }

        ICollectionGenericObjects<DrawningAirCraft>? collection = _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            _logger.LogError("Не удалось инициализировать коллекцию");
            return;
        }

        switch (comboBoxSelectorCompany.Text)
        {
            case "Ангар":
                _company = new AirCraftAngar(pictureBox.Width, pictureBox.Height, collection);
                break;
        }

        panelCompanyTools.Enabled = true;
        RerfreshListBoxItems();
    }

    /// <summary>
	/// Обработка нажатия "Сохранение"
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.SaveData(saveFileDialog.FileName);
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

    /// <summary>
	/// Обработка нажатия "Загрузка"
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.LoadData(openFileDialog.FileName);
                MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Загрузка из файла: {filename}", openFileDialog.FileName);
                RerfreshListBoxItems();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

    /// <summary>
	/// Сортировка по типу
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonSortByType_Click(object sender, EventArgs e)
    {
        CompareAirCraft(new DrawingAirCraftCompareByType());
    }

    /// <summary>
    /// Сортировка по цвету
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonSortByColor_Click(object sender, EventArgs e)
    {
        CompareAirCraft(new DrawningAirCraftCompareByColor());
    }

    /// <summary>
    /// Сортировка по сравнителю
    /// </summary>
    /// <param name="comparer">Сравнитель объектов</param>
    private void CompareAirCraft(IComparer<DrawningAirCraft?> comparer)
    {
        if (_company == null)
        {
            return;
        }

        _company.Sort(comparer);
        pictureBox.Image = _company.Show();
    }
}